#
# main.py - interface for the main page
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>

import datetime
from flask import Blueprint, render_template, redirect, flash, url_for, request
from flask.ext.wtf import Form, TextField, SelectField, BooleanField, PasswordField
from flask.ext.wtf.html5 import IntegerField
from wtforms.ext.sqlalchemy.orm import model_form
from flask.ext.login import login_user, logout_user, login_required, current_user, AnonymousUser

from blockerbugs import app, db, login_manager
from blockerbugs.models.bug import Bug
from blockerbugs.models.release import Release
from blockerbugs.models.settings import Settings
from blockerbugs.models.user import User

admin = Blueprint('admin', __name__)

# forms
class CurrentReleaseForm(Form):
    release = SelectField(u'Current Release', coerce=int)

ReleaseForm = model_form(Release, Form)

class LoginForm(Form):
    username = TextField(u'Username')
    password = PasswordField(u'Password')


# handle login stuff
@login_manager.user_loader
def load_user(userid):
    app.logger.debug("getting info for user %s" % str(userid))
    user = User.query.get(userid)
    if user:
        return user
    else:
        return AnonymousUser


@admin.route('/login', methods=['GET', 'POST'])
def login():
    login_form = LoginForm()

    if login_form.validate_on_submit():
        user = User.query.filter_by(username=login_form.username.data).first()
        if user and user.check_password(login_form.password.data):
            login_user(user)

            app.logger.info('Successful login for user %s' % login_form.username.data)
            flash('Logged In Successfully!')
            return redirect(request.args.get('next') or url_for('.admin_index'))
        else:
            app.logger.info('FAILED login for user %s' % login_form.username.data)
            flash('Login Failed! Please Try again!')

    return render_template('login.html', form=login_form)


@admin.route('/logout')
@login_required
def logout():
    app.logger.info('logout for user %s' % current_user.username)
    logout_user()
    return redirect(url_for('main.display_current'))


def get_current_release():
    current_release = Release.query.filter_by(current=True).first()
    if current_release:
        return current_release.name
    else:
        return ''


def set_current_release(new_release_id):
    new_release = Release.query.filter_by(id = new_release_id).first()
    current_releases = Release.query.filter_by(current = True).all()
    for current_release in current_releases:
        if current_release.id == new_release_id:
            continue
        current_release.current = False
        db.session.add(current_release)
    new_release.current = True
    db.session.add(new_release)
    db.session.commit()


@admin.route('/admin/main', methods=("GET","POST"))
@login_required
def admin_index():
    if app.debug:
        app.logger.debug('rendering index')
    releases = Release.query.all()
    current_relform = CurrentReleaseForm()
    current_relform.release.choices = [(r.id, "Fedora %d %s" % (r.number, r.version)) for r in Release.query.filter_by(active=True).all()]
    if current_relform.validate_on_submit():
        app.logger.debug('got %s as the new current release' % current_relform.release.data)
        set_current_release(current_relform.release.data)
        return redirect(url_for(".admin_index"))
    else:
        app.logger.debug('relform did not validate for request type %s' % request.method)
    return render_template('admin/main.html', form=current_relform, releases = releases)


@admin.route('/admin/release', methods=("GET","POST"))
@login_required
def release():
    releases = Release.query.all()
    current_release = Release.query.filter_by(current=True).first()
    if current_release:
        current = current_release.name
    else:
        current = ''

    newform = ReleaseForm(request.form)
    if newform.validate_on_submit():
        new_release = Release(newform.number.data, newform.version.data,
                                newform.blocker_tracker.data,
                                newform.accepted_tracker.data, newform.name.data,
                                active=newform.active.data, current=False)
        db.session.add(new_release)
        db.session.commit()
        return redirect(url_for(".admin_index"))
    return render_template('admin/add_release.html', releases = releases,
                                            current=current, newform=newform)


@admin.route('/admin/release/<int:number>/<version>', methods=('GET', 'POST'))
@login_required
def modify_release(number, version):
    release = Release.query.filter_by(number=number, version=version).first()
    rel_form = ReleaseForm(request.form, release)
    if rel_form.validate_on_submit():
        rel_form.populate_obj(release)
        db.session.add(release)
        db.session.commit()
        return redirect(url_for('.admin_index'))
    return render_template('admin/modify_release.html', form=rel_form)

