#
# main.py - interface for the main page
#
# Copyright 2012, Red Hat, Inc
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Authors:
#   Tim Flink <tflink@redhat.com>

from flask import Blueprint, render_template, redirect, url_for
import datetime

from blockerbugs import app
from blockerbugs.models.bug import Bug
from blockerbugs.models.release import Release

main = Blueprint('main', __name__)

def get_recent_modifications(releaseid):
    now = datetime.datetime.utcnow()
    modcutoff = now - datetime.timedelta(1)
    recentmods = Bug.query.filter_by(release_id=releaseid).filter(Bug.date_changed > modcutoff).all()
    return [bug.bugid for bug in recentmods]


def get_release_bugs(release):
    bugz = {'Accepted Blockers':[], 'Proposed Blockers':[], 'Accepted NTH':[], 'Proposed NTH':[]}

    bugz['Accepted Blockers'] = Bug.query.filter_by(release=release,
                                                    accepted_blocker=True,
                                                    active=True).all()
    bugz['Proposed Blockers'] = Bug.query.filter_by(release=release,
                                                    proposed_blocker=True,
                                                    active=True).all()
    bugz['Accepted NTH'] = Bug.query.filter_by(release=release,
                                                    accepted_nth=True,
                                                    active=True).all()
    bugz['Proposed NTH'] = Bug.query.filter_by(release=release,
                                                    proposed_nth=True,
                                                    active=True).all()
    return bugz

def get_current_release():
    current_release = Release.query.filter_by(current=True).first()
    # if no current release is set, just grab one
    if not current_release:
        current_release = Release.query.first()
    return current_release


def get_release_info(release):
    return {'number' : release.number,
            'phase' : release.version.title(),
            'criteria_url' : 'http://fedoraproject.org/wiki/Fedora_%d_%s_Release_Criteria' % (release.number, release.version.title()),
            'last_updated' : release.last_updated.strftime('%Y-%m-%d %H:%M:%S UTC')
            }


@main.route('/')
def index():
    if app.debug:
        app.logger.debug('rendering index')
    releases = Release.query.filter_by(active=True).all()
    return render_template('index.html', title = 'Fedora Blocker Bugs',
                                         releases = releases)


@main.route('/<int:num>/<release_name>')
def display_blockers(num, release_name):
    release = Release.query.filter_by(number=num, version=release_name).first()
    release_info = get_release_info(release)
    bugz = get_release_bugs(release)
    recent_bugs = get_recent_modifications(release.id)
    return render_template('basic_blockers.html',
                            buglists=bugz,
                            recent = recent_bugs,
                            info = release_info,
                            title = "Fedora %s %s Blocker Bugs" % (release_info['number'], release_info['phase']))


@main.route('/<int:num>/<release_name>/irc')
def display_irc_blockers(num, release_name):
    release = Release.query.filter_by(number=num, version=release_name).first()
    bugz = get_release_bugs(release)
    return render_template('irc_format.txt', buglists=bugz)


@main.route('/current')
def display_current():
    release = get_current_release()
    release_info = get_release_info(release)
    bugz = get_release_bugs(release)
    recent_bugs = get_recent_modifications(release.id)
    return render_template('basic_blockers.html',
                            buglists=bugz,
                            recent = recent_bugs,
                            info = release_info,
                            title = "Fedora %s %s Blocker Bugs" % (release_info['number'], release_info['phase']))


@main.route('/current/irc')
def display_current_irc():
    release = get_current_release()
    return redirect(url_for('.display_irc_blockers', num=release.number, release_name=release.version))
